<?php
//@codingStandardsIgnoreStart
class ControllerModuleRoistat extends Controller {

    public function index() {
        $this->load->model('setting/setting');
        $settings = $this->model_setting_setting->getSetting('roistat');

        if (!array_key_exists('roistat_password', $settings) || !array_key_exists('roistat_username', $settings)) {
            echo 'Data of authorization aren\'t set';
            exit;
        }

        $action_method = $this->getRequest('action');
        $auth_token    = $this->getRequest('token');

        $username = $settings['roistat_username'];
        $password = $settings['roistat_password'];

        if ((string)$auth_token !== (string)md5($username . $password)) {
            echo 'Incorrect token';
            exit;
        }

        if ($action_method === 'export') {
            $output = $this->loadOrders();
        } elseif ($action_method === 'export_clients') {
            $output = $this->loadClients();
        } elseif ($action_method === 'lead') {
            $output = $this->createLead();
        } else {
            $output = array('status' => 'error', 'message' => 'Action is undefined');
        }

        $this->response->addHeader('Content-Type: application/json; charset=utf8');
        $this->response->setOutput(json_encode($output));
    }

    /**
     * @return array
     */
    private function loadOrders() {
        $result = array('orders' => array(), 'statuses' => array());

        $modifed_date = (int)$this->getRequest('date', time() - 31*24*60*60);

        $language_id = $this->config->get('config_language_id');
        $store_id    = $this->config->get('config_store_id');

        $this->load->language('module/roistat');

        $statuses = $this->db->query("SELECT * FROM `" . DB_PREFIX . "order_status` WHERE `language_id` = '" . $language_id . "';");
        $sql = "SELECT `order_id`, `date_added`, `date_modified`, `order_status_id`, `total`,
                       `roistat_visit`, `customer_id`, `shipping_method`, `payment_method`, `affiliate_id`
                FROM `" . DB_PREFIX . "order`
                WHERE `store_id` = '" . $store_id . "'
                AND `date_modified` > {$modifed_date};";
        $orders = $this->db->query($sql);

        foreach ($statuses->rows as $status) {
            $result['statuses'][] = array(
                'id'   => $status['order_status_id'],
                'name' => $status['name'],
            );
        }
        foreach ($orders->rows as $order) {
            $newOrder = array(
                'id'          => $order['order_id'],
                'date_create' => $order['date_added'],
                'date_update' => $order['date_modified'],
                'status'      => $order['order_status_id'],
                'price'       => $order['total'],
                'visit'       => $order['roistat_visit'],
                'client_id'   => $order['customer_id'],
                'fields' => array(),
            );
            if (array_key_exists('shipping_method', $order) && $order['shipping_method'] !== '') {
                $newOrder['fields'][$this->language->get('entry_shipping_method')] = $order['shipping_method'];
            }
            if (array_key_exists('payment_method', $order) && $order['payment_method'] !== '') {
                $newOrder['fields'][$this->language->get('entry_payment_method')] = $order['payment_method'];
            }
            if (array_key_exists('affiliate_id', $order) && $order['affiliate_id'] !== '') {
                $newOrder['fields'][$this->language->get('text_affiliate')] = $order['affiliate_id'];
            }
            $result['orders'][] = $newOrder;
        }
        return $result;
    }

    /**
     * @return array
     */
    private function createLead() {
        $customer_id = 0;
        $store_name      = $this->config->get('config_name');
        $store_id        = $this->config->get('config_store_id');
        $store_url       = $this->config->get('config_url');
        $language_id     = $this->config->get('config_language_id');
        $order_status_id = $this->config->get('config_order_status_id');

        $title   = $this->db->escape($this->getRequest('title'));
        $name    = $this->db->escape($this->getRequest('name'));
        $email   = $this->db->escape($this->getRequest('email'));
        $phone   = $this->db->escape($this->getRequest('phone'));
        $visit   = $this->db->escape($this->getRequest('visit'));
        $comment = $this->db->escape($this->getRequest('text'));
        $first_name = ($name === '' || $name === null) ? $title : $name;

        $found_customer_id = $this->getCustomerIdByEmail($email);
        if ($found_customer_id) {
            $customer_id = $found_customer_id;
        }

        $sql = "INSERT INTO `" . DB_PREFIX . "order`
                        SET `order_status_id` = '{$order_status_id}',
                            `language_id` = '{$language_id}',
                            `telephone` = '{$phone}',
                            `email` = '{$email}',
                            `firstname` = '{$first_name}',
                            `customer_id` = '{$customer_id}',
                            `store_url` = '{$store_url}',
                            `store_name` = '{$title} ({$store_name})',
                            `store_id` = '{$store_id}',
                            `comment` = '{$comment}',
                            `roistat_visit` = '{$visit}',
                            `date_added` = NOW(),
                            `date_modified` = NOW()";
        $this->db->query($sql);
        return array('status' => 'ok', 'order_id' => $this->db->getLastId());
    }

    /**
     * @param string $email
     * @return array
     */
    private function getCustomerIdByEmail($email) {
        $query = $this->db->query("SELECT DISTINCT * FROM " . DB_PREFIX . "customer WHERE LCASE(email) = '{$email}';");

        if (count($query->row) > 0) {
            return $query->row['customer_id'];
        } else {
            return false;
        }
    }

    /**
     * @return array
     */
    private function loadClients() {
        return array();
    }

    /**
     * @param string $key
     * @param mixed $default
     * @return string
     */
    private function getRequest($key, $default = null) {
        return array_key_exists($key, $this->request->request) ? $this->request->request[$key] : $default;
    }

    /**
     * For event 'post.order.add'
     *
     * @param int $order_id
     */
    public function saveOrderVisit($order_id) {
        if (array_key_exists('roistat_visit', $this->request->cookie) && !$this->isAdmin()) {
            $visit_id = $this->request->cookie['roistat_visit'];

            $this->updateVisitByOrderId($order_id, $visit_id);
        }
    }

    /**
     * For AJAX in order_form
     */
    public function updateVisit() {
        $this->load->language('module/roistat');

        if ($this->isAdmin()) {
            $order_id = $this->getRequest('order_id');
            $visit_id = $this->db->escape($this->getRequest('visit'));
            $this->updateVisitByOrderId($order_id, $visit_id);

            $output = array('success' => $this->language->get('text_edit_visit_success'));
        } else {
            $output = array('error' => $this->language->get('text_edit_visit_error'));
        }
        $this->response->addHeader('Content-Type: application/json; charset=utf8');
        $this->response->setOutput(json_encode($output));
    }

    /**
     * @param int $order_id
     * @param string $visit
     */
    private function updateVisitByOrderId($order_id, $visit) {
        $this->db->query("UPDATE `" . DB_PREFIX . "order` SET `roistat_visit` = '" . $visit . "' WHERE `order_id` = '" . (int)$order_id . "';");
    }

    /**
     * @return bool
     */
    private function isAdmin() {
        $user = new User($this->registry);
        return (bool)$user->isLogged();
    }
}
//@codingStandardsIgnoreEnd